// ----------------------------------------------------------------------------
// $Id: CommentFilter.java,v 1.2 1999/07/12 08:32:11 schreine Exp $
// filter for '//' comments
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;

import java.io.*;

// we need this class because filtering of '//' comments does not work
// in StreamTokenizer, if '/' is a word character (stupid implementation)
public final class CommentFilter extends Reader
{
  // underlying reader
  private Reader reader;

  // --------------------------------------------------------------------------
  // comment fileter from underlying reader
  // --------------------------------------------------------------------------
  public CommentFilter(Reader _reader)
  {
    reader = _reader;
  }

  // special characters
  private final int EOF = -1;
  private final int NL = 10;
  private final int CR = 13;

  // we can't use nlString because in Microsoft's Internet Explorer
  // read() returns erroneously NL instead of CR+NL
  // private String nlString = System.getProperty("line.separator");
  // private int nlLen = nlString.length();
  
  // not a character
  private final int EMPTY = -2;
  private int ch = EMPTY;

  // --------------------------------------------------------------------------
  // return next character, -1 if EOF
  // --------------------------------------------------------------------------
  public int read() throws IOException
  {
    // deliver not yet delivered character
    if (ch != EMPTY)
    {
      int ch0 = ch;
      if (ch != EOF)
	ch = EMPTY;
      return ch0;
    }

    // no character to be delivered
    while (true)
    {
      ch = reader.read();
      if (ch == EOF) return ch;
      if (ch != '/') 
	{
	  int ch0 = ch;
	  ch = EMPTY;
	  return ch0;
	}

      // '/' was read
      ch = reader.read();
      if (ch != '/')
	return '/';

      // '//' was read
      do 
      {
	ch = reader.read();
	if (ch == EOF) return ch;
      }
      while (ch != NL && ch != CR);

      // recognize DOS end of line
      if (ch == CR)
      {
	ch = reader.read();
	if (ch != NL) return ch;
      }
    }
  }
  
  // end of file has been encountered
  private boolean eof = false;

  // --------------------------------------------------------------------------
  // fill buffer with characters, return number of chars read, EOF if EOF
  // --------------------------------------------------------------------------
  public int read(char[] buffer, int offset, int length) throws IOException
  {
    if (eof) return EOF;
    int i = offset;
    int n = offset+length;
    while (i < n)
    {
      int ch = read();
      if (ch == EOF) break;
      buffer[i] = (char)ch;
      i++;
    }
    eof = i < n;
    return i-offset;
  }

  // --------------------------------------------------------------------------
  // close reader
  // --------------------------------------------------------------------------
  public void close() throws IOException
  {
    reader.close();
  }
}

// ----------------------------------------------------------------------------
// $Id: CommentFilter.java,v 1.2 1999/07/12 08:32:11 schreine Exp $
// ----------------------------------------------------------------------------
