// ----------------------------------------------------------------------------
// $Id: Context.java,v 1.7 1999/07/12 08:32:11 schreine Exp $
// context holding variable bindings
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;
import java.util.*;

public final class Context
{
  // we use public (not private) for member vars because of jdk/netscape bugs

  // the model used for static class references
  public static Context context = null;

  // --------------------------------------------------------------------------
  // set context used for static class references
  // --------------------------------------------------------------------------
  public static void setContext(Context _context)
  {
    context = _context;
  }

  // --------------------------------------------------------------------------
  // the static class interface
  // --------------------------------------------------------------------------
  public static void reset()
  {
    context._reset();
  }
  public static void begin(String variable, Value value)
  {
    context._begin(variable, value);
  }
  public static void end()
  {
    context._end();
  }
  public static Value get(String variable)
  {
    return context._get(variable);
  }

  // --------------------------------------------------------------------------
  // the private members
  // --------------------------------------------------------------------------

  // out set of bindings
  public Vector bindings = new Vector();
  
  // pair of variable and value
  private final static class Binding
  {
    String variable;
    Value value;
    public Binding(String _variable, Value _value)
    {
      variable = _variable;
      value = _value;
    }
  }

  // --------------------------------------------------------------------------
  // reset the context
  // --------------------------------------------------------------------------
  private void _reset()
  {
    bindings = new Vector();
  }

  // --------------------------------------------------------------------------
  // begin a new context with a binding 'variable=value'
  // --------------------------------------------------------------------------
  private void _begin(String variable, Value value)
  {
    Binding binding = new Binding(variable, value);
    bindings.addElement(binding);
  }

  // --------------------------------------------------------------------------
  // end the current context
  // --------------------------------------------------------------------------
  private void _end()
  {
    bindings.removeElementAt(bindings.size()-1);
  }
  
  // --------------------------------------------------------------------------
  // return the value of 'variable' in the current context (null, if none)
  // --------------------------------------------------------------------------
  private Value _get(String variable)
  {
    for (int i=bindings.size()-1; i>=0; i--)
      {
	Binding binding = (Binding)bindings.elementAt(i);
	if (binding.variable.equals(variable))
	  return binding.value;
      }
    return null;
  }
}
// ----------------------------------------------------------------------------
// $Id: Context.java,v 1.7 1999/07/12 08:32:11 schreine Exp $
// ----------------------------------------------------------------------------
