// ----------------------------------------------------------------------------
// $Id: DefinedFunction.java,v 1.3 1999/06/29 12:46:40 schreine Exp $
// an explicitly defined function
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;

public final class DefinedFunction implements Function
{
  private String string;
  private ConstrainedVariable[] params;
  private Term term;

  // --------------------------------------------------------------------------
  // function 'name(params) := term'
  // --------------------------------------------------------------------------
  public DefinedFunction(String _name, ConstrainedVariable[] _params, 
			 Term _term)
  {
    string = _name;
    params = _params;
    term = _term;
  }

  // --------------------------------------------------------------------------
  // the name of the function
  // --------------------------------------------------------------------------
  public String name()
  {
    return string;
  }

  // --------------------------------------------------------------------------
  // the number of function parameters
  // --------------------------------------------------------------------------
  public int arity()
  {
    return params.length;
  }

  // ------------------------------------------------------------------------
  // apply function to arguments
  // ------------------------------------------------------------------------
  public Value apply(Value[] arguments) throws EvalException
  {
    if (Model.checkGuard())
    {
      for (int i=0; i<params.length; i++)
	params[i].check(arguments[i]);
    }
    for (int i=0; i<params.length; i++)
      Context.begin(params[i].name(), arguments[i]);
    Value result = term.eval();
    for (int i=0; i<params.length; i++)
      Context.end();
    return result;
  }
}
// ----------------------------------------------------------------------------
// $Id: DefinedFunction.java,v 1.3 1999/06/29 12:46:40 schreine Exp $
// ----------------------------------------------------------------------------
