// ----------------------------------------------------------------------------
// $Id: DefinedPredicate.java,v 1.3 1999/06/29 12:46:40 schreine Exp $
// an explicitly defined predicate
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;

public final class DefinedPredicate implements Predicate
{
  private String string;
  private ConstrainedVariable[] params;
  private Formula formula;

  // --------------------------------------------------------------------------
  // the predicate 'name(params) :<=> formula'
  // --------------------------------------------------------------------------
  public DefinedPredicate(String _name, ConstrainedVariable[] _params, 
			  Formula _formula)
  {
    string = _name;
    params = _params;
    formula = _formula;
  }

  // --------------------------------------------------------------------------
  // the name of the predicate
  // --------------------------------------------------------------------------
  public String name()
  {
    return string;
  }

  // --------------------------------------------------------------------------
  // the number of predicate parameters
  // --------------------------------------------------------------------------
  public int arity()
  {
    return params.length;
  }

  // ------------------------------------------------------------------------
  // apply predicate to arguments
  // ------------------------------------------------------------------------
  public boolean apply(Value[] arguments) throws EvalException
  {
    if (Model.checkGuard())
    {
      for (int i=0; i<params.length; i++)
	params[i].check(arguments[i]);
    }
    for (int i=0; i<params.length; i++)
      Context.begin(params[i].name(), arguments[i]);
    boolean result = formula.eval();
    for (int i=0; i<params.length; i++)
      Context.end();
    return result;
  }
}
// ----------------------------------------------------------------------------
// $Id: DefinedPredicate.java,v 1.3 1999/06/29 12:46:40 schreine Exp $
// ----------------------------------------------------------------------------
