// ----------------------------------------------------------------------------
// $Id: Main.java,v 1.27 1999/07/12 14:26:55 schreine Exp $
// main program
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;

public final class Main extends Applet
{
  // the help URL
  public final static String url = 
  "http://www.risc.uni-linz.ac.at/software/formal";

  // --------------------------------------------------------------------------
  // executed when run as application
  // --------------------------------------------------------------------------
  public static void main(String[] args)
  {
    // not in applet mode
    Control control = new Control(false, null, System.in, System.out);

    // set model and context
    Model.setModel(new Model());
    Context.setContext(new Context());

    control.println("> Logic Evaluator 1.0 (c) 1999, Wolfgang Schreiner.");
    control.println("> Type 'help;' for help.");

    Reader reader = new InputStreamReader(control.getInputStream());
    new Interpreter(control).run(reader);

    control.println("> End of input stream.");
  }

  // --------------------------------------------------------------------------
  // text shown as applet info
  // --------------------------------------------------------------------------
  public String getAppletInfo()
  {
    return "A Logic Evaluator";
  }

  // --------------------------------------------------------------------------
  //  text shown as parameter info
  // --------------------------------------------------------------------------
  public String[][] getParameterInfo()
  {
    String info[][] = {
      {"exec",  "String", "input to be executed"},
      {"input", "String", "text of input field"},
      {"screen", "String", "set if screen is to be displayed"},
      {"swidth", "Integer", "width of screen (default 50)"},
      {"sheight", "Integer", "height of screen (default 50)"},
      {"lines", "Integer", "number of output lines"}};
    return info;
  }

  // where plots are drawn (may be null)
  protected Screen screen;

  // where output is displayed
  protected TextArea output;
  protected TextAreaStream outputStream;

  // where input is entered
  protected TextField input;
  protected TextFieldStream inputStream;

  // where help is triggered
  protected Button help;

  // our control object
  protected Control control;

  // --------------------------------------------------------------------------
  // executed when run as applet
  // --------------------------------------------------------------------------
  public void init()
  {    
    // determine number of output lines (only used if screen is created)
    int lines = 6;
    String lstring = getParameter("lines");
    try
    {
      if (lstring != null)
	lines = new Integer(lstring).intValue();
    }
    catch(NumberFormatException e)
    {
      System.out.println("illegal parameter format");
    }

    // create components
    output = new TextArea("", lines, 80, TextArea.SCROLLBARS_VERTICAL_ONLY);
    input = new TextField("");
    help = new Button("?");

    // create screen
    screen = newScreen();
    int offset = screen == null ? 0 : 1;

    // layout output field above input field
    GridBagLayout gridbag = new GridBagLayout();
    setLayout(gridbag);

    // screen can be resized in both dimensions
    if (screen != null)
    {
      GridBagConstraints c = new GridBagConstraints();
      c.gridx = 0;
      c.gridy = 0;
      c.gridwidth = 2;
      c.gridheight = 1;
      c.weightx = 1.0;
      c.weighty = 1.0;
      c.fill = GridBagConstraints.BOTH;
      gridbag.setConstraints(screen, c);
      screen.setBackground(Color.white);
    }

    // output can be in general resized in both dimensions
    // if screen is given, resize is only horizontally possible
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = 0;
    c.gridy = offset;
    c.gridwidth = 2;
    c.gridheight = 1;
    c.weightx = 1.0;
    c.anchor = GridBagConstraints.SOUTH;
    if (screen == null)
    {
      c.fill = GridBagConstraints.BOTH;
      c.weighty = 1.0;
    }
    else
    {
      c.fill = GridBagConstraints.HORIZONTAL;
      c.weighty = 0.0;
    }
    gridbag.setConstraints(output, c);
    output.setFont(new Font("Courier", Font.PLAIN, 12));
    output.setBackground(Color.white);
    output.setForeground(Color.black);
    output.setEditable(false);

    // input can be resized horizontally only
    GridBagConstraints d = new GridBagConstraints();
    d.gridx = 0;
    d.gridy = 1+offset;
    d.gridwidth = 1;
    d.gridheight = 1;
    d.weightx = 1.0;
    d.weighty = 0.0;
    d.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(input, d);
    input.setFont(new Font("Courier", Font.BOLD, 12));
    input.setBackground(Color.white);
    input.setForeground(Color.black);
    String text = getParameter("input");
    if (text != null)
      input.setText(text);

    // panel can be not resized
    GridBagConstraints e = new GridBagConstraints();
    e.gridx = 1;
    e.gridy = 1+offset;
    e.gridwidth = 1;
    e.gridheight = 1;
    e.weightx = 0.0;
    e.weighty = 0.0;
    e.fill = GridBagConstraints.VERTICAL;
    gridbag.setConstraints(help, e);
    help.setFont(new Font("Courier", Font.PLAIN, 12));
    help.setBackground(Color.lightGray);
    help.setForeground(Color.black);

    // add components
    if (screen != null)
      add(screen);
    add(output);
    add(input);
    add(help);
 
    // in applet mode
    inputStream = new TextFieldStream();
    outputStream = new TextAreaStream(output);
    control = new Control(true, screen, inputStream, outputStream);
  }

  // --------------------------------------------------------------------------
  // return screen for visualization
  // --------------------------------------------------------------------------
  private Screen newScreen()
  {
    if (getParameter("screen") == null)
      return null;    
    int width = 50;
    int height = 50;
    try
    {
      String xstring = getParameter("swidth");
      if (xstring != null)
	width = new Integer(xstring).intValue();
      String ystring = getParameter("sheight");
      if (ystring != null)
	height = new Integer(ystring).intValue();
    }
    catch(NumberFormatException e)
    {
      System.out.println("illegal parameter format");
    }
    return new Screen(width, height);
  }

  // the one thread that is executing on current static data
  protected static Thread executing = null;

  // used as our lock object
  protected static Object key = new Object();

  // our model and context
  protected Model model = new Model();
  protected Context context = new Context();

  // --------------------------------------------------------------------------
  // mutual exclusion lock, sets 'executing' to 'thread'
  // sets model and context for evaluation
  // --------------------------------------------------------------------------
  protected void lock(Thread thread)
  {
    synchronized (key)
    {
      while (executing != null)
      {
	try
	{
	  key.wait();
	}
	catch(InterruptedException e)
	{
	}
      }
      executing = thread;
      Model.setModel(model);
      Context.setContext(context);
    }
  }

  // --------------------------------------------------------------------------
  // mutual exclusion lock, frees 'executing'
  // --------------------------------------------------------------------------
  protected void unlock()
  {
    synchronized (key)
    {
      executing = null;
      key.notify();
    }
  }

  // --------------------------------------------------------------------------
  // interpreter runs in separate thread
  // --------------------------------------------------------------------------
  private class InterpreterThread extends Thread
  {
    private Interpreter interpreter;
    private InputStream input;

    public InterpreterThread(Interpreter _interpreter, InputStream _input)
    {
      interpreter = _interpreter;
      input = _input;
      setPriority(getPriority()+1);
      this.start(); 
    }

    public void run()
    {
      // parameters are processed
      String exec = getParameter("exec");
      if (exec != null)
      {
	exec = filter(exec);
	control.println(exec);
	Reader reader = new StringReader(exec + "\n");
	lock(this);
	interpreter.run(reader);
	unlock();
      }

      // thread is suspendend, gets lower priority than user interface threads
      suspend();
      setPriority(getPriority()-2);

      // when resumed, user input is processed
      Reader reader = new InputStreamReader(input);
      interpreter.run(reader);
      control.println("> end of input stream.");
    }

    // filter \n sequences and replace by newline characters
    private String filter(String text)
    {
      int n = text.length();
      StringBuffer buffer = new StringBuffer();
      int start = 0;
      int search=0;
      while (true)
      {
	int i = text.indexOf('\\', search);
	if (i == -1)
	{
	  buffer.append(text.substring(start, n));
	  break;
	}
	else if (i < n-1 && text.charAt(i+1) == 'n')
	{
	  buffer.append(text.substring(start, i));
	  buffer.append("\n");
	  start = i+2;
	  search = start;
	  if (start >= n) break;
	}
	else
	  search = i+1;
      }
      return buffer.toString();      
    }
  }

  // our interpreter thread
  protected Thread thread = null;

  // --------------------------------------------------------------------------
  // applet resumes execution
  // --------------------------------------------------------------------------
  public void start()
  {
    // must wait for initialization to be complete
    if (thread == null)
      {
	// interpreter for this applet
	Interpreter interpreter = new Interpreter(control);

	// start thread and make sure it is in suspended mode
	Thread t = new InterpreterThread(interpreter, 
					 control.getInputStream());
	Thread.yield();

	// record thread
	thread = t;

	// register event handlers
	input.addFocusListener(new FocusHandler());
	input.addActionListener(new InputHandler());
	help.addActionListener(new ButtonHandler());
      }
  }

  // --------------------------------------------------------------------------
  // applet suspends execution
  // --------------------------------------------------------------------------
  public void stop()
  {
    // thread is suspended when focus is lost
  }

  // --------------------------------------------------------------------------
  // applet terminates execution
  // --------------------------------------------------------------------------
  public void destroy()
  {
    if (thread != null)
    {
      thread.stop();
      thread = null;
    }
  }

  // --------------------------------------------------------------------------
  // handle input event
  // --------------------------------------------------------------------------
  private class InputHandler implements ActionListener
  {
    // triggered when ENTER is pressed
    public void actionPerformed(ActionEvent event)
    {
      String text = input.getText();
      control.println(text);
      inputStream.append(text + "\n");
      input.setText("");
    }
  }

  // light gray
  protected Color focusColor = new Color(165, 165, 165);

  // --------------------------------------------------------------------------
  // handle focus event
  // --------------------------------------------------------------------------
  private class FocusHandler implements FocusListener
  {
    // triggered when focus is gained
    public void focusGained(FocusEvent event)
    {
      if (thread != null && thread != executing)
      {
	lock(thread);
	thread.resume();
	input.setBackground(focusColor);
      }
    }

    // triggered when focus is lost
    public void focusLost(FocusEvent event)
    {
      if (thread != null && thread == executing)
      {
	input.setBackground(Color.white);
	thread.suspend();
	unlock();
      }
    }
  }

  // --------------------------------------------------------------------------
  // handle button event
  // --------------------------------------------------------------------------
  private class ButtonHandler implements ActionListener
  {
    // triggered when button is pressed
    public void actionPerformed(ActionEvent event)
    {
      if (control.isApplet())
      {
	try
	{
	  URL url0 = new URL(url+"/language.html");
	  AppletContext context = getAppletContext();
	  context.showDocument(url0, "logic");
	}
	catch (MalformedURLException e)
	{
	  control.println("> malformed URL '" + url + "'.");
	  control.abort();
	}
      }
    }
  }

  // --------------------------------------------------------------------------
  // display output in text area
  // --------------------------------------------------------------------------
  private class TextAreaStream extends OutputStream
  {
    private TextArea output;
    private boolean pending = false;
    private String nl = System.getProperty("line.separator");
    private int len = nl.length();

    public TextAreaStream(TextArea _output)
    {
      output = _output;
    }

    public void write(byte[] b, int offset, int length)
    {
      if (pending)
	{
	  output.append(nl);
	  pending = false;
	}      
      if (length >= len && nl.equals(new String(b, offset+length-len, len)))
	{
	  output.append(new String(b, offset, length-len));
	  pending = true;
	}
      else
	output.append(new String(b, offset, length));
    }

    public void write(byte[] b)
    {
      write(b, 0, b.length);
    }

    public void write(int b)
    {
      byte[] a = new byte[1];
      a[0] = (byte)b;
      write(a);
    }
  }

  // --------------------------------------------------------------------------
  // receive input from text field
  // --------------------------------------------------------------------------
  public class TextFieldStream extends InputStream
  {
    private String text = "";
    private int index = 0;

    public synchronized void append(String _text)
    {
      text = text+_text;
      notify();
    }

    public synchronized int available()
    {
      return text.length()-index;
    }

    private synchronized void getText()
    {
      while (index == text.length())
      {
	try
	{
	  wait();
	}
	catch(InterruptedException e)
	{
	}
      }
    }

    private synchronized void clearText()
    {
      if (index == text.length())
      {
	index = 0;
	text = "";
      }
    }

    public synchronized int read(byte b[], int offset, int length) 
    {
      getText();
      int a = available();
      if (a < length) length = a;
      byte[] b0 = text.substring(index, index+length).getBytes();
      index = index+length;
      clearText();
      if (b != null)
	System.arraycopy(b0, 0, b, offset, length);
      return length;
    }

    public synchronized int read()
    {
      getText();
      int ch = (int)text.charAt(index);
      index++;
      clearText();
      return ch;
    }
  }
}
// ----------------------------------------------------------------------------
// $Id: Main.java,v 1.27 1999/07/12 14:26:55 schreine Exp $
// ----------------------------------------------------------------------------
