// ----------------------------------------------------------------------------
// $Id: RecursiveFunction.java,v 1.1 1999/07/12 08:32:11 schreine Exp $
// a recursively defined function
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;

public final class RecursiveFunction implements Function
{
  private String string;
  private ConstrainedVariable[] params;
  private Term indterm;
  private Term term;

  // induction value (-1 signals not called yet)
  private int indvalue = -1;

  // --------------------------------------------------------------------------
  // function 'name(params) recursive indterm := term'
  // --------------------------------------------------------------------------
  public RecursiveFunction(String _name, ConstrainedVariable[] _params, 
			   Term _indterm, Term _term)
  {
    string = _name;
    params = _params;
    indterm = _indterm;
    term = _term;
  }

  // --------------------------------------------------------------------------
  // the name of the function
  // --------------------------------------------------------------------------
  public String name()
  {
    return string;
  }

  // --------------------------------------------------------------------------
  // the number of function parameters
  // --------------------------------------------------------------------------
  public int arity()
  {
    return params.length;
  }

  // ------------------------------------------------------------------------
  // apply function to arguments
  // ------------------------------------------------------------------------
  public Value apply(Value[] arguments) throws EvalException
  {
    if (Model.checkGuard())
    {
      for (int i=0; i<params.length; i++)
	params[i].check(arguments[i]);      
    }
    for (int i=0; i<params.length; i++)
      Context.begin(params[i].name(), arguments[i]);
    int oldvalue = indvalue;
    if (Model.checkGuard())
    {
      Value v = indterm.eval();
      if (!(v instanceof Nat))
	throw new EvalException("induction value " + v.getString() + 
				" of function " + string + "/" + arity() +
				" is not a natural number");
      int newvalue = ((Nat)v).intValue();
      if (oldvalue != -1 && newvalue >= oldvalue)
	throw new EvalException("new induction value " + newvalue + 
				" of function " + string + "/" + arity() +
				" is not less than old value " + oldvalue);
      indvalue = newvalue;
    }
    Value result = null;
    try
    {
      result = term.eval();
    }
    catch(EvalException e)
    {
      indvalue = oldvalue;
      throw(e);
    }
    indvalue = oldvalue;
    for (int i=0; i<params.length; i++)
      Context.end();
    return result;
  }
}
// ----------------------------------------------------------------------------
// $Id: RecursiveFunction.java,v 1.1 1999/07/12 08:32:11 schreine Exp $
// ----------------------------------------------------------------------------
