// ----------------------------------------------------------------------------
// $Id: RecursivePredicate.java,v 1.1 1999/07/12 08:32:11 schreine Exp $
// a recursively defined predicate
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;

public final class RecursivePredicate implements Predicate
{
  private String string;
  private ConstrainedVariable[] params;
  private Term indterm;
  private Formula formula;

  // induction value (-1 signals not called yet)
  private int indvalue = -1;

  // --------------------------------------------------------------------------
  // predicate 'name(params) recursive indterm := formula'
  // --------------------------------------------------------------------------
  public RecursivePredicate(String _name, ConstrainedVariable[] _params, 
			   Term _indterm, Formula _formula)
  {
    string = _name;
    params = _params;
    indterm = _indterm;
    formula = _formula;
  }

  // --------------------------------------------------------------------------
  // the name of the predicate
  // --------------------------------------------------------------------------
  public String name()
  {
    return string;
  }

  // --------------------------------------------------------------------------
  // the number of predicate parameters
  // --------------------------------------------------------------------------
  public int arity()
  {
    return params.length;
  }

  // ------------------------------------------------------------------------
  // apply predicate to arguments
  // ------------------------------------------------------------------------
  public boolean apply(Value[] arguments) throws EvalException
  {
    if (Model.checkGuard())
    {
      for (int i=0; i<params.length; i++)
	params[i].check(arguments[i]);      
    }
    for (int i=0; i<params.length; i++)
      Context.begin(params[i].name(), arguments[i]);
    int oldvalue = indvalue;
    if (Model.checkGuard())
    {
      Value v = indterm.eval();
      if (!(v instanceof Nat))
	throw new EvalException("induction value " + v.getString() + 
				" of predicate " + string + "/" + arity() +
				" is not a natural number");
      int newvalue = ((Nat)v).intValue();
      if (oldvalue != -1 && newvalue >= oldvalue)
	throw new EvalException("new induction value " + newvalue + 
				" of predicate " + string + "/" + arity() +
				" is not less than old value " + oldvalue);
      indvalue = newvalue;
    }
    boolean result = false;
    try
    {
      result = formula.eval();
    }
    catch(EvalException e)
    {
      indvalue = oldvalue;
      throw(e);
    }
    indvalue = oldvalue;
    for (int i=0; i<params.length; i++)
      Context.end();
    return result;
  }
}
// ----------------------------------------------------------------------------
// $Id: RecursivePredicate.java,v 1.1 1999/07/12 08:32:11 schreine Exp $
// ----------------------------------------------------------------------------
