// ----------------------------------------------------------------------------
// $Id: Screen.java,v 1.4 1999/07/12 08:32:11 schreine Exp $
// panel for 2D grid visualization
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;

import java.awt.*;
import java.awt.event.*;

public final class Screen extends Panel
{
  // dimension in logical pixels
  private int width;
  private int height;
  private Color[][] color;

  // current dimension
  private Dimension dimension;

  // for double buffering
  private Image image;

  // --------------------------------------------------------------------------
  // create screen of size _width/_height
  // --------------------------------------------------------------------------
  public Screen(int _width, int _height)
  {
    width = _width;
    height = _height;
    color = new Color[width][height];    
    setBackground(Color.white);
    init();
  }

  // --------------------------------------------------------------------------
  // clear screen
  // --------------------------------------------------------------------------
  public void clear()
  {
    init();
  }

  // --------------------------------------------------------------------------
  // set pixel 'x/y' with color 'c'
  // --------------------------------------------------------------------------
  public void set(int x, int y, Color c)
  {
    if (x < width && y < height)
      color[x][y] = c;
  }

  // --------------------------------------------------------------------------
  // flush output to screen
  // --------------------------------------------------------------------------
  public void flush()
  {
    paint(getGraphics());
  }

  // --------------------------------------------------------------------------
  // 
  // internal methods below
  // 
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  // initialize pixel array
  // --------------------------------------------------------------------------
  private void init()
  {
    Color bgColor = getBackground();
    for (int i=0; i<width; i++)
      for (int j=0; j<height; j++)
	color[i][j] = bgColor;
  }

  // --------------------------------------------------------------------------
  // called to create new image with dimension `d`
  // --------------------------------------------------------------------------
  private void redraw(Dimension d)
  {
    dimension = d;
    int w = d.width/width;
    int h = d.height/height;
    if (w < 2) w = 2;
    if (h < 2) h = 2;
    image = createImage(d.width, d.height);
    Graphics g = image.getGraphics();
    for (int i=0; i<width; i++) 
      for (int j=0; j<height; j++)
      {
	g.setColor(color[i][j]);
	g.fillRect(i*w, d.height-(j+1)*h, w-1, h-1);
      }
    g.setColor(Color.gray);
    g.drawRect(0, 0, d.width-1, d.height-1);
  }

  // --------------------------------------------------------------------------
  // draws the screen
  // --------------------------------------------------------------------------
  public void paint(Graphics g)
  {
    Dimension d = getSize();
    if (dimension == null || d.equals(dimension))
      redraw(d);
    g.drawImage(image, 0, 0, this);
  }
}
// ----------------------------------------------------------------------------
// $Id: Screen.java,v 1.4 1999/07/12 08:32:11 schreine Exp $
// ----------------------------------------------------------------------------
