// ----------------------------------------------------------------------------
// $Id: Set.java,v 1.6 1999/07/12 08:32:11 schreine Exp $
// a set domain
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;
import java.util.*;

public final class Set implements Domain
{
  // protected (not private) because of netscape bug
  protected Vector vector;

  // --------------------------------------------------------------------------
  // the set represented by `_vector`
  // --------------------------------------------------------------------------
  private Set(Vector _vector)
  {
    vector = _vector;
  }

  // --------------------------------------------------------------------------
  // the empty set
  // --------------------------------------------------------------------------
  public Set()
  {
    this(new Vector());
  }

  // --------------------------------------------------------------------------
  // an unshared duplicate of this set
  // --------------------------------------------------------------------------
  public Set duplicate()
  {
    return new Set((Vector)vector.clone());
  }

  // --------------------------------------------------------------------------
  // returns true iff `element` is in set
  // --------------------------------------------------------------------------
  public boolean hasElement(Value element)
  {
    int n = vector.size();
    for (int i=0; i<n; i++)
      {
	if (element.equals((Value)vector.elementAt(i)))
	  return true;
      }
    return false;
  }

  // --------------------------------------------------------------------------
  // an element of the set (null, if set is empty)
  // --------------------------------------------------------------------------
  public Value element()
  {
    if (vector.size() == 0)
      return null;
    else
      return (Value)vector.elementAt(0);
  }

  // --------------------------------------------------------------------------
  // is `set` a subset of this set?
  // --------------------------------------------------------------------------
  private boolean isSubset(Set set)
  {
    int n = vector.size();
    for (int i=0; i<n; i++)
      {
	Value element = (Value)vector.elementAt(i);
	if (!set.hasElement(element)) return false;
      }
    return true;
  }

  // --------------------------------------------------------------------------
  // does `value` equal this set?
  // --------------------------------------------------------------------------
  public boolean equals(Value value)
  {
    if (!(value instanceof Set)) return false;
    Set set = (Set)value;
    return this.isSubset(set) && set.isSubset(this);
  }

  // --------------------------------------------------------------------------
  // the printed representation of this set
  // --------------------------------------------------------------------------
  public String getString()
  {
    StringBuffer buffer = new StringBuffer();
    buffer.append("{");
    int n = vector.size();
    for (int i=0; i<n; i++)
      {
	Value element = (Value)vector.elementAt(i);
	buffer.append(element.getString());
	if (i < n-1) buffer.append(", ");
      }
    buffer.append("}");
    return buffer.toString();
  }

  // --------------------------------------------------------------------------
  // destructively add `element` to this set
  // --------------------------------------------------------------------------
  public void addElement(Value element)
  {
    if (!hasElement(element))
      vector.addElement(element);
  }

  // --------------------------------------------------------------------------
  // a set iterator
  // --------------------------------------------------------------------------
  private class SetIterator implements Iterator
  {
    int index;
    
    public SetIterator()
    {
      index = 0;
    }

    public boolean hasNext()
    {
      return index < vector.size();
    }

    public Value next()
    {
      Value element = (Value)vector.elementAt(index);
      index++;
      return element;
    }
  }

  // --------------------------------------------------------------------------
  // an iterator for this set
  // --------------------------------------------------------------------------
  public Iterator getIterator()
  {
    return new SetIterator();
  }
}
// ----------------------------------------------------------------------------
// $Id: Set.java,v 1.6 1999/07/12 08:32:11 schreine Exp $
// ----------------------------------------------------------------------------
