// ----------------------------------------------------------------------------
// $Id: TypedVariable.java,v 1.4 1999/07/12 12:17:36 schreine Exp $
// a pair of a variable and a term (denoting a domain)
//
// (c) 1999, Wolfgang Schreiner, see file COPYRIGHT
// http://www.risc.uni-linz.ac.at/software/formal
// ----------------------------------------------------------------------------
package Logic;

public final class TypedVariable implements 
  IteratorVariable, ConstrainedVariable
{
  private String string;
  private Term term;

  public TypedVariable(String _string, Term _term)
  {
    string = _string;
    term = _term;
  }

  public String name()
  {
    return string;
  }

  public Term domain()
  {
    return term;
  }

  // --------------------------------------------------------------------------
  // returns an iterator for the domain (for the universe, if no domain given)
  // --------------------------------------------------------------------------
  public Iterator iterator() throws EvalException
  {
    Value value = term.eval();
    if (!(value instanceof Domain))
      throw new EvalException
	("value " + value.getString() + " does not denote a domain");
    return ((Domain)value).getIterator();    
  }

  // --------------------------------------------------------------------------
  // throws exception if value does not match variable constraints
  // --------------------------------------------------------------------------
  public void check(Value value) throws EvalException
  {
    Domain d = (Domain)(term.eval());
    if (!d.hasElement(value))
      throw new EvalException
	("value " + value.getString() + 
	 " is not in domain of parameter " + string);
  }
}
// ----------------------------------------------------------------------------
// $Id: TypedVariable.java,v 1.4 1999/07/12 12:17:36 schreine Exp $
// ----------------------------------------------------------------------------
